@@ -38,6 +38,7 @@ gtk.window_set_default_icon_name(ICON_NAME)
 
 def loadUIFile(name, root = None):
     ui = gtk.Builder()
+    ui.set_translation_domain(DOMAIN)
     ui.add_from_file(os.path.join(UI_DIR, name))
     return ui
 
@@ -151,44 +152,6 @@ class GLibTimer(glchess.ui.Timer):
 class GtkUI(glchess.ui.UI):
     """
     """
-    # The Gtk+ GUI
-    _gui               = None
-    
-    # The time stored for animation
-    __lastTime         = None
-    __animationTimer   = None
-    
-    # The Gtk+ list model of the available player types
-    __playerModel      = None
-
-    # The about dialog open
-    __aboutDialog      = None
-
-    # Dictionary of save game dialogs keyed by view
-    __saveGameDialogs  = None
-
-    __renderGL         = False
-    openGLInfoPrinted  = False
-
-    # TODO
-    __joinGameDialogs  = None
-    __networkGames     = None
-    
-    __defaultWhiteAI   = None
-    __defaultBlackAI   = None
-
-    __attentionCounter = 0
-
-    whiteTimeString    = '∞'
-    blackTimeString    = '∞'
-    
-    # The window width and height when unmaximised and not fullscreen
-    width              = None
-    height             = None
-    isFullscreen       = False
-    isMaximised        = False
-    
-    view               = None
 
     def __init__(self, feedback):
         """Constructor for a GTK+ glChess GUI"""
@@ -197,21 +160,30 @@ class GtkUI(glchess.ui.UI):
         self.__networkGames = {}
         self.newGameDialog = None
         self.loadGameDialog = None
+        self.__aboutDialog = None
         self.__saveGameDialogs = {}
         self.__joinGameDialogs = []
-        
-        # Set the message panel to the tooltip style
-        # (copied from Gedit)
-        # In Gtk+ 2.11+ (I think) tip_window is now private so skip if it's not there (bug #459740)
-        tooltip = gtk.Tooltips()
-        tooltip.force_window()
-        if hasattr(tooltip, 'tip_window') and tooltip.tip_window != None:
-            tooltip.tip_window.ensure_style()
-            self._tooltipStyle = tooltip.tip_window.get_style()
-        else:
-            self._tooltipStyle = None
-        self._tooltipWidgetsDrawn = {}
-        
+
+        # The time stored for animation
+        self.__lastTime         = None
+        self.__animationTimer   = None
+    
+        self.__renderGL         = False
+        self.openGLInfoPrinted  = False
+
+        self.__attentionCounter = 0
+
+        self.whiteTimeString    = '∞'
+        self.blackTimeString    = '∞'
+    
+        # Theo window width and height when unmaximised and not fullscreen
+        self.width              = None
+        self.height             = None
+        self.isFullscreen       = False
+        self.isMaximised        = False
+    
+        self.view               = None
+
         self._gui = loadUIFile('glchess.ui')
         self._gui.connect_signals(self)
         
@@ -243,38 +215,14 @@ class GtkUI(glchess.ui.UI):
         
         # Watch for config changes
         for key in ['show_toolbar', 'show_history', 'fullscreen',
-                    'show_3d', 'show_3d_smooth', 'show_comments', 'show_numbering',
-                    'show_move_hints',
-                    'width', 'height',
+                    'show_3d', 'show_3d_smooth', 'piece_style', 'show_comments',
+                    'show_numbering', 'show_move_hints', 'width', 'height',
                     'move_format', 'promotion_type', 'board_view',
                     'enable_networking']:
             glchess.config.watch(key, self.__applyConfig)
 
     # Public methods
     
-    def setTooltipStyle(self, widget):
-        """Set a widget to be in the tooltip style.
-        
-        'widget' is the widget to modify.
-        """
-        if self._tooltipStyle is None:
-            return
-        widget.set_style(self._tooltipStyle)
-        widget.connect("expose_event", self._on_tooltip_expose_event)
-        widget.queue_draw()
-
-    def _on_tooltip_expose_event(self, widget, event):
-        """Gtk+ callback"""
-        allocation = widget.allocation
-        widget.style.paint_flat_box(widget.window, gtk.STATE_NORMAL, gtk.SHADOW_OUT, None, widget, "tooltip",
-                                    allocation.x, allocation.y, allocation.width, allocation.height)
-                                    
-        # The first draw is corrupt for me so draw it twice.
-        # Bonus points to anyone who tracks down the problem and fixes it
-        if not self._tooltipWidgetsDrawn.has_key(widget):
-            self._tooltipWidgetsDrawn[widget] = True
-            widget.queue_draw()
-    
     def watchFileDescriptor(self, fd):
         """Extends ui.UI"""
         self._watches[fd] = gobject.io_add_watch(fd, gobject.IO_IN | gobject.IO_PRI | gobject.IO_HUP | gobject.IO_ERR, self.__readData)
@@ -311,10 +259,6 @@ class GtkUI(glchess.ui.UI):
         """
         iter = self.__playerModel.append()
         self.__playerModel.set(iter, 0, name, 1, 'stock_notebook', 2, name)
-        
-        # Get the human to play against this AI
-        if self.__defaultBlackAI is None:
-            self.__defaultBlackAI = name
 
     def setView(self, title, feedback, isPlayable = True):
         """Extends ui.UI"""
@@ -391,7 +335,7 @@ class GtkUI(glchess.ui.UI):
         """        
         # Load configuration
         for name in ['show_toolbar', 'show_history', 'show_3d', 'show_3d_smooth',
-                     'show_comments', 'show_numbering', 'show_move_hints',
+                     'piece_style', 'show_comments', 'show_numbering', 'show_move_hints',
                      'move_format', 'promotion_type', 'board_view', 'maximised',
                      'enable_networking']:
             try:
@@ -578,6 +522,9 @@ Please contact your system administrator to resolve these problems, until then y
                 value = False
             self.view.feedback.showSmooth(value)
                 
+        elif name == 'piece_style':
+            pass
+
         elif name == 'show_comments':
             self.view.setShowComments(value)
 
@@ -749,7 +696,7 @@ Please contact your system administrator to resolve these problems, until then y
         """
         """
         enable = self.view is not None and self.view.isPlayable
-        for widget in ('save_game_button', 'menu_save_item', 'menu_save_as_item'):
+        for widget in ('menu_save_item', 'menu_save_as_item'):
             self.__getWidget(widget).set_sensitive(enable)
 
         combo = self.__getWidget('history_combo')
